
//*******************************************************************
//
//  UNIT:     G r 3 2 _ E x t          05.11.2003,  07.11.2006
//
//  Expansion Graphic Unit for graphics32 library v1.7.x, 1.8.x
//
//  Copyright (c) 2006 by Gerd Platl
//
//  --- functions and procedures ---
//
//  ResetColor                    reset bitmap color without changing alpha
//  SelectBitmapResampling        select resampling method for bitmap stretching
//  CopyLine                      copy horizontal bitmap line
//  MakeTransparentBackground     make bitmap background transparent
//  SetTransparentBackground      set bitmap background transparent
//  ResizeBitmap32                resize bitmap
//  ZoomBitmap32
//
//  BitmapXOR                     invert pixel values of bitmap with given mask
//  Invert                        invert rgb and alpha pixel values of given bitmap
//  InvertRGB                     invert rgb pixel values of given bitmap
//  InvertRGBA                    invert rgb and alpha pixel values of given bitmap
//  InvertAlpha                   invert alpha pixel values of given bitmap
//  GreyscaleFilter               greyscale image applying a 'colour lens filter'
//  SepiaFilter                   sepia color filtering
//  PosterizeFilter               posterize given bitmap
//  GlassTileFilter               filter like looking through a wall of glass tiles
//
//  SaveBitmapToBMP               save bitmap as standand windows BMP file
//  SaveBitmapToJPG               save bitmap as JPEG picture file
//  SaveBitmap32ToJPG             save bitmap32 as JPEG picture file
//  PrintBitmap32                 call printing dialog and print bitmap32
//
//  SelectBitmap32                select drawing bitmap
//  DrawPolygon                   draw polygon
//  CalcEllipsePoints             calculate non rotated ellipse polygon points
//  CalcRotatedEllipsePoints      calculate rotated ellipse polygon points
//  DrawEllipse                   draw ellipse as polygon curve
//  CalcCubicCurve                calculate polygon points of cubic curve
//  DrawCubicCurve                draw cubic curve
//
//  CopyTiles                     copy tile with over the whole bitmap
//  DrawTiles                     fill destination bitmap with given tile bitmap 
//  DrawScaledTiles               fill dest. bitmap with tiles scaled to rect before
//  DrawChessboard                fill whole bitmap with a chessboard pattern 
//
//  --- global variable ---
//
//  activeBitmap: tBitmap32;      active drawing bitmap
//
//*******************************************************************
{$I comp_opt.inc}
{$R-}

unit Gr32_Ext;

{$ifopt D+}
  {$define xtest1   main function test}
  {$define test2   sub function test}
{$endif}

INTERFACE

uses  Windows, Gr32, Gr32_Blend, Gr32_Image, GR32_Resamplers, Gr32_Polygons;

type  pColorBGRA = ^tColorBGRA;        // matches Gr32.tColor32
      tColorBGRA = packed record       // color32 with byte values
        b,g,r, a: byte;   end;         // see Gr32.TColor32Entry

type  pColorRGBA = ^tColorRGBA;        // matches Graphics.tColor
      tColorRGBA = packed record       // color with byte values
        r,g,b, a: byte;   end;         // red green blue alpha

type  tLongColor = record              // color with longint values
        r,g,b, a: longint;   end;      // as 24.8 fix point values

type  tStretchFilter                   // resampling filters for >= v1.8
      = (sfNearest, sfLinear, sfDraft,
         sfBoxKernel, sfLinearKernel, sfCosineKernel, sfSplineKernel,
         sfCubicKernel, sfMitchellKernel, sfLanczosKernel, sfGaussianKernel,
         sfBlackmanKernel, sfHannKernel, sfHammingKernel, sfSinshKernel,
         sfHermiteKernel);

//--- tBitmap32 routines ---

procedure ResetColor (bm: TBitmap32; color: tColor32);

procedure SelectBitmapResampling (src: tBitmap32; sf: tStretchFilter);

procedure CopyLine (src, dest: tBitmap32; x1,y1, x2,y2, w: integer);

procedure MakeTransparentBackground (src, dest: tBitmap32);
procedure SetTransparentBackground (src, dest: tBitmap32;
                                    bgColor: tColor32;
                                    maxDist: integer = 100;
                                    minOpaque: byte = 32);

procedure ResizeBitmap32 (var src: tBitmap32; w,h: integer);
procedure ZoomBitmap32 (src, dst: tBitmap32; delta: integer);

//--- GR32_Filter.pas extensions ---

procedure BitmapXOR (dst, src: TBitmap32; mask: longWord);
procedure Invert (dst, src: TBitmap32);
procedure InvertRGB (dst, src: TBitmap32);
procedure InvertRGBA (dst, src: TBitmap32; R,G,B,A: boolean);
procedure InvertAlpha (dst, src: TBitmap32);

procedure GreyscaleFilter (dst, src: TBitmap32; const filter: TColor32 = clWhite32);

procedure SepiaFilter (src, dst: TBitmap32; sepiaIntensity: integer = 30);

procedure PosterizeFilter (src, dst: tBitmap32; amount: integer);

procedure GlassTileFilter (src, dst: TBitmap32; const tileWidth, tileHeight: Integer);

//--- picture file routines ---

function SaveBitmapToBMP (const bitmap: tBitmap32;
                          const filename: string): boolean;

(*
function SaveBitmapToJPG (const bitmap: tBitmap;
                          const filename: string;
                          const quality: integer = 95;
                          const progEncoding: boolean = false): boolean;
*)

function SaveBitmap32ToJPG (const bitmap: tBitmap32;
                            const filename: string;
                            const quality: integer = 95;
                            const progEncoding: boolean = false): boolean;

procedure PrintBitmap32 (const bitmap: tBitmap32;
                         const title: string;
                         const printTitle: boolean;
                         const position: word;
                         const frame: word = 5);

function GetFittingRect (const src, dst: TRect;
                         const position: word;
                         var rect: TRect): boolean;

function GetScaledRect (const outerA, innerA: tRect;  sizeA: TPoint;
                        var sizeB: TPoint): boolean;

//--- drawing routines ---

procedure SelectBitmap32 (bitmap: tBitmap32);

procedure DrawPolygon (const bitmap: tBitmap32;
                       const points: tArrayOfFixedPoint;
                       const color: tColor32;
                       const closed: boolean;
                       const antialiased: boolean;
                       const filled: boolean);

procedure CalcEllipsePoints (const mx,my, a,b: single;
                             const edges: cardinal;
                             var points: TArrayOfFixedPoint);

procedure CalcRotatedEllipsePoints (const mx,my, a,b, angle: single;
                                    const edges: cardinal;
                                    const addPoints: boolean;
                                    var points: TArrayOfFixedPoint);

procedure DrawEllipse (mx,my, a,b, angle: single;
                       color: tColor32;
                       edges: cardinal;
                       thickness: cardinal;
                       antialiased: boolean;
                       filled: boolean);

procedure CalcCubicCurve (const points: TArrayOfFixedPoint;
                          const steps: cardinal;
                          const closed: boolean;
                          var curve: TArrayOfFixedPoint);

procedure DrawCubicCurve (const points: tArrayOfFixedPoint;
                          const steps: cardinal;
                          const color: tColor32;
                          const closed: boolean;
                          const antialiased: boolean;
                          const filled: boolean;
                          const thickness: byte = 1);

procedure CopyTiles (dx,dy: integer; bitmap: tBitmap32);
procedure DrawTiles (src, dest: tBitmap32; x,y: integer);
procedure DrawScaledTiles (src, dest: tBitmap32; rect: tRect);
procedure DrawChessboard (bm: tBitmap32; dx,dy: integer; c1,c2: tColor32);
procedure ImageChessboard (img: tImgView32; dx,dy: integer; col1, col2: tColor32);

//--- tPalette32 routines ---

procedure SpreadColor32 (var pal: tArrayOfColor32; first, last: tColorRGBA);
procedure SetSpreadPalette32 (var pal: TArrayOfColor32;
                              var spreadValues: array of tColorRGBA);

var   activeBitmap: tBitmap32 = nil;   // active drawing bitmap

//*********************************************************
IMPLEMENTATION

uses sysutil2,  Classes, Forms, Graphics, SysUtils, Printers, JPEG, Gr32_Filters;

type  float = extended;                { 10 bytes (FPU-format)}
const deg = Pi / 180.0;                {1 / Radiant = 0.01745329252...}

var   hLine32Bm: tBitmap32 = nil;      {32 bit work line}
      outline: tPolygon32;

//*********************************************************
// reset color information (RGB values) of a bitmap
// without changing alpha values !
//   (analog to Gr32.ResetAlpha)
// input:      bm      source and destination bitmap
//             color   new fill color
//---------------------------------------------------------
procedure ResetColor (bm: TBitmap32; color: tColor32);
var
  ni: integer;
  cp: PColor32;
begin
  color := color and $00FFFFFF;
  cp := @bm.Bits[0];
  for ni := 0 to (bm.height*bm.width)-1 do
  begin
    cp^ := cp^ and $FF000000 or color;
    inc(cp);
  end;
end;
//*********************************************************
// select resampling method for bitmap stretching
// input:      src: tBitmap32;       source bitmap
//             sf: tStretchFilter;   resampling methode
//---------------------------------------------------------
procedure SelectBitmapResampling (src: tBitmap32; sf: tStretchFilter);
begin
  if ord(sf) <= 3
  then TBitmap32ResamplerClass(ResamplerList[ord(sf)]).Create(src)
  else begin
    TKernelResampler.Create(src);
    TKernelResampler(src.Resampler).Kernel
      := TCustomKernelClass(KernelList[ord(sf)-3]).Create;
  end;
end;
//*********************************************************
// copy w pixels of horizontal line from src position x1,y1
// to destination position x2,y2
//---------------------------------------------------------
procedure CopyLine (src, dest: tBitmap32; x1,y1, x2,y2, w: integer);

var srcRect, dstRect: tRect;
begin
  srcRect.Left := x1;    srcRect.Right := x1 + w;
  srcRect.Top := y1;     srcRect.Bottom := y1;
  dstRect.Left := x2;    dstRect.Right := x2 + w;
  dstRect.Top := y2;     dstRect.Bottom := y2;

  dest.Draw (dstRect, srcRect, src);
end;

//*********************************************************
// set alpha value of every pixel to 0, which is equal
// to the background color defined as pixel[0,0]
//---------------------------------------------------------
(*
procedure MakeTransparentBackground (src, dest: tBitmap32);
var    xi, yi, xc: integer;   col: tColor32;
       sl: pColor32Array;
begin
  dest.Assign(src);
  col := dest.Pixel[0,0];              // transparent pixel color
  xc := dest.Width - 1;
  for yi := 0 to dest.Height - 1
  do begin
    sl := ColorRGB.pColor32Array(dest.ScanLine[yi]);
    for xi := 0 to xc                  // line loop
    do if sl[xi] = col
    then sl[xi] := 0;                  // set transparent pixel
  end;
end;
*)
//---------------------------------------------------------
procedure MakeTransparentBackground (src, dest: tBitmap32);
var   ni: integer;   col: tColor32;
    {  S: PColor32Entry; }    D: PColor32;        //ToDo: ungetestet

begin
  dest.Assign(src);                    // copy src -> dest
//  S := @src.Bits[0];
  D := @Dest.Bits[0];
  col := dest.Pixel[0,0];              // transparent pixel color
  for ni := 0 to dest.Width * dest.Height - 1
  do begin
    if d^ = col                        // transparent pixel ?
    then d^ := 0;                      // set transparent pixel
  {  Inc(S);}   Inc(D);                  // point to next pixel
  end;
end;

//*********************************************************
// * set transparency of every pixel to 0, which is equal
//   to the given background color.
// * interpolate transparency for every pixel which color differences
//   is less then maxDiff.
// note: on complete transparency rgb value of the pixel are deleted
// input:   src, dest     source and destination bitmap
//          color         background color (complete transparent)
//          maxDist       optional max. distance to an opaque pixel color
//          minOpaque     optional minimal opaqueness
//                        if pixel opaqueness <= minOpaque -> pixel invisible
// see also:  Gr32-Filters: AlphaToGrayscale, IntensityToAlpha
//---------------------------------------------------------
(*
procedure SetTransparentBackground (src, dest: tBitmap32;
                                    color: tColor32;
                                    maxDist: integer = 100;
                                    minOpaque: byte = 32);
const  invBlack: tColorBGRA = (a:0);
var    ni, dist: integer;
       bgc: tColorBGRA;           // backgroundcolor
       pix: pColorBGRA;           // ARGB pixel
begin
  if dest <> src then dest.Assign(src);  // copy bitmap
  maxDist := sqr(maxDist);
  bgc := tColorBGRA(color);
  pix := pColorBGRA(dest.PixelPtr[0, 0]);
  for ni := 0 to (dest.Height * dest.Width - 1)
  do begin
    dist := sqr(pix.r - bgc.r)    // calculate color distance
          + sqr(pix.g - bgc.g)
          + sqr(pix.b - bgc.b);
    if dist < maxDist             // make pixel transparent ?
    then begin                    // change alpha value
      pix.a := MulDiv (dist, 255, maxDist);
      if pix.a < minOpaque        // optimize pixel opaqueness ?
      then pix^ := invBlack;      // set pixel complete invisible
    end;
    inc(pix);                     // proceed to the next pixel
  end;
end;
*)
//*********************************************************
procedure SetTransparentBackground (src, dest: tBitmap32;
                                    bgColor: tColor32;
                                    maxDist: integer = 100;
                                    minOpaque: byte = 32);
const  invBlack: TColor32Entry = (a:0);
var    ni, dist: integer;
       bgc: TColor32Entry;        // background color
       pix: PColor32Entry;        // ARGB pixel
begin
  if dest <> src then dest.Assign(src);     // copy bitmap
  maxDist := sqr(maxDist);
  bgc := TColor32Entry (bgColor);
  pix := PColor32Entry (dest.PixelPtr[0, 0]);
  for ni := 0 to (dest.Height * dest.Width - 1)
  do begin
    dist := sqr(pix.r - bgc.r)    // calculate color distance
          + sqr(pix.g - bgc.g)
          + sqr(pix.b - bgc.b);
    if dist < maxDist             // make pixel transparent ?
    then begin                    // change alpha value
      pix.a := MulDiv (dist, 255, maxDist);
      if pix.a < minOpaque        // optimize pixel opaqueness ?
      then pix^ := invBlack;      // set pixel complete invisible
    end;
    inc(pix);                     // proceed to the next pixel
  end;
end;
//*********************************************************
// resize bitmap
// input:    src: tBitmap32;     source bitmap
//           w,h: integer;         new width and height
//                                 if w=0 calculate proportional width
//                                 if h=0 calculate proportional height
//---------------------------------------------------------
procedure ResizeBitmap32 (var src: tBitmap32; w,h: integer);
var
  tempBm: tBitmap32;
begin
  if w = 0 then w := src.Width  * h div src.Height;
  if h = 0 then h := src.Height * w div src.Width;
  tempBm := tBitmap32.Create;
  tempBm.SetSize (w, h);
  src.DrawTo (tempBm, MakeRect(0,0,w,h));
  src.Free;
  src := tempBm;
end;
//*********************************************************
// zoom bitmap in or out
// note:     call SelectBitmapResampling before zoom calling
//           to select stretch filtering
// input:    src: tBitmap32;   source bitmap
//           dst: tBitmap32;   destination bitmap
//           delta: integer;   zoom factor  >0: zoom in
//                                          <0: zoom out
//---------------------------------------------------------
procedure ZoomBitmap32 (src, dst: tBitmap32; delta: integer);

var   srcRect, dstRect: TRect;
begin
  if delta = 0 then exit;
  if delta > 0
  then begin                           // zoom in +
    srcRect := MakeRect(delta,delta, src.Width-1-delta, src.height-1-delta);
    dstRect := MakeRect(0,0, src.Width-1, src.height-1);
  end else begin                       // zoom out -
    delta := -delta;
    srcRect := MakeRect(0,0, src.Width-1, src.height-1);
    dstRect := MakeRect(delta,delta, src.Width-1-delta, src.height-1-delta);
  end;
  src.DrawTo (dst, dstRect, srcRect);
end;

//--- GR32_Filter.pas extensions ---

//*********************************************************
// invert pixel values of bitmap with given mask
// input:      mask     pixel bit mask
//---------------------------------------------------------
procedure BitmapXOR (dst, src: TBitmap32; mask: longWord);
var
  I: Integer;
  dp, sp: PColor32;
begin
  CheckParams(dst, src);
  dst.SetSize(src.Width, src.Height);
  if src.Empty then exit;
  dp := @dst.Bits[0];
  sp := @src.Bits[0];
  for I := 0 to src.Width * src.Height - 1 do
  begin
    dp^ := sp^ xor mask;
    Inc(sp);   Inc(dp);
  end;
  dst.Changed;
end;

//*********************************************************
// invert rgb and alpha pixel values of given bitmap
// note:   same procedure as in GR32_Filters.pas
//---------------------------------------------------------
procedure Invert (dst, src: TBitmap32);
begin
  BitmapXOR (dst, src, $FFFFFFFF);
end;

//*********************************************************
// invert rgb pixel values of given bitmap
//---------------------------------------------------------
procedure InvertRGB (dst, src: TBitmap32);
begin
  BitmapXOR (dst, src, $00FFFFFF);
end;

//*********************************************************
// invert alpha pixel values of given bitmap
//---------------------------------------------------------
procedure InvertAlpha (dst, src: TBitmap32);
begin
  BitmapXOR (dst, src, $FF000000);
end;

//*********************************************************
// invert rgb and alpha pixel values of given bitmap
// input:      R,G,B,A: boolean   pixel part flags
//---------------------------------------------------------
procedure InvertRGBA (dst, src: TBitmap32; R,G,B,A: boolean);
begin
  BitmapXOR (dst, src, longWord(B) * $FF
                    or longWord(G) * $FF00
                    or longWord(R) * $FF0000
                    or longWord(A) * $FF000000);
end;

(*
from:  thequinns@worldonline.co.uk   3.4.2006  news.graphics32.org

Below is a procedure to greyscale a Bitmap32 image, applying a 'colour lens filter'.

With 3 gauge bars (r, g, b), any colour 'filter' can be approximated,
when making a grey scale image from a colour photograph.

Colour filters were (still are?) used all the time in serious Black and
White film photography. Some notes based on a 1960's camera filter guide:
A yellow filter is 'ideal for use all the time'. 'Lightly darkens' blue
sky with landscape and scenery shots. Provides 'bright expression' of people.
Orange: 'stresses the colour sensitively and provides contrast' and
        'gives a natural reproduction' of scenery, buildings etc
Red:    A 'typical contrast increasing filter' that 'reproduces mountains,
        sea and sky with dynamic contrasts'
Green (Yellow Green): '..is effective with portraits of people',
        'reproduces skins and lips in natural tones'
*)
//*********************************************************
// greyscale image applying a 'colour lens filter'
// input:   src, dest     source and destination bitmap
//          filter        filter color
//---------------------------------------------------------
procedure GreyscaleFilter (dst, src: TBitmap32;
                           const filter: TColor32 = clWhite32);
var
   fc, sp: PColor32Entry;   dp: PColor32;
   ni, grey, sc: integer;
   redLUT, greenLUT, blueLUT: array[0..255] of Integer;
begin
  if src.Height = 0 then exit;
  // Gr32_Filters.CheckParams (dst, src);
  dst.SetSize (src.Width, src.Height);
  fc := @filter;
  sp := @src.Bits[0];
  dp := @dst.Bits[0];

  // if filter color is black bitmap is black too
  if (Filter = clBlack32) then begin
    dst.Clear;
    dst.Changed;
    Exit;
  end;

  // build the look up tables
  for ni := 0 to 255 do begin
    redLUT[ni]   := MulDiv (fc.R, ni, 255);
    greenLUT[ni] := MulDiv (fc.G, ni, 255);
    blueLUT[ni]  := MulDiv (fc.B, ni, 255);
  end;

  // greyscale the bitmap
  sc := fc.R + fc.G + fc.B;
  for ni := 0 to dst.Width * dst.Height - 1 do begin
    if sp.ARGB = 0
    then dp^ := 0
    else begin
      grey := ((redLUT[sp.R] + greenLUT[sp.G] + blueLUT[sp.B]) shl 8) div sc;
      dp^ := integer(sp.ARGB and $FF000000) + grey shl 16 + grey shl 8 + grey;
    end;
    Inc(sp);   Inc(dp);
  end;
  dst.Changed;
end;
//*********************************************************
// convert given bitmap to sepia color
// input:   src, dest       source and destination bitmap
//          sepiaIntensity  	expedient values 10 .. 80
//---------------------------------------------------------
procedure SepiaFilter (src, dst: TBitmap32; sepiaIntensity: integer = 30);
var
  ni, redIntensity: integer;
  sp, dp: PColor32Entry;
begin
  if dst <> src then dst.Assign(src);
  redIntensity := MulDiv (sepiaIntensity, 16, 9);
  sp := @src.Bits[0];
  dp := @dst.Bits[0];
  for ni := 0 to dst.Width * dst.Height - 1
  do begin
    dp.b := Intensity (sp.ARGB);
    dp.r := ClampByte (sp.b + redIntensity);
    dp.g := ClampByte (sp.b + sepiaIntensity);
    Inc(sp);   Inc(dp);
  end;
  dst.Changed;
end;
//*********************************************************
// posterize given bitmap (color stairs)
// input:   src, dest     source and destination bitmap
//          amount        amount of posterization 0 .. 255
//---------------------------------------------------------
procedure PosterizeFilter (src, dst: tBitmap32; amount: integer);
var    xi, yi, xc: integer;
       sl: pColor32Array;    // scan line pointer
       pix: pColorBGRA;      // pixel pointer
begin
  if amount < 1 then amount := 1
  else if amount > 255 then amount := 255;
  if dst <> src then dst.Assign(src);
  xc := dst.Width - 1;
  for yi := 0 to dst.Height - 1
  do begin
    sl := pColor32Array(dst.ScanLine[yi]);
    for xi := 0 to xc
    do begin
      pix := @sl[xi];
      pix.r := (pix.r div amount) * amount;
      pix.g := (pix.g div amount) * amount;
      pix.b := (pix.b div amount) * amount;
    end;
  end;
  dst.Changed;
end;
//*********************************************************
// glass tiles filter image
// source and destination bimtmap can be the same.
// for docu look at http://docs.gimp.org/en/plug-in-glasstile.html
// input:   src, dest     source and destination bitmap
//          tileWidth     tile width in pixel
//                        w <= 1 means no effect in x direction
//          tileHeight    tile height in pixel
//                        h <= 1 means no effect in y direction
//---------------------------------------------------------
procedure GlassTileFilter (src, dst: TBitmap32;
                           const tileWidth, tileHeight: Integer);
var
  xi, sx, xts, xm, xoffs: Integer;
  yi, sy, yts, ym, yoffs: Integer;
  SRows, DRows: array [0 .. 2047] of PColor32Array;
  tempBm: tBitmap32;
begin
  // check and set picture size
  if src.Height = 0 then exit;
  if   dst.Width <> src.Width
  then dst.Width := src.Width;
  if   dst.Height <> src.Height
  then dst.Height := src.Height;

  // create a temporary bitmap, if source bitmap = destination bitmap
  tempBm := nil;
  try
  if src = dst then begin
    tempBm := tBitmap32.Create;
    tempBm.Assign(src);
    src := tempBm;
  end;

  // create scanline pointer list
  for yi := 0 to src.Height - 1 do
  begin
    SRows[yi] := src.ScanLine[yi];
    DRows[yi] := dst.ScanLine[yi];
  end;

  xts := -tileWidth;
  yts := -tileHeight;
  ym := 0;
  yoffs := 0;
  for yi := 0 to src.Height - 1 do
  begin
    if tileHeight < 2
    then sy := yi
    else begin
      sy := ym + yoffs;
      if   sy < 0
      then sy := 0;
      if   sy > (src.Height - 1)
      then sy := src.Height - 1;
    end;

    xm := 0;
    xoffs := 0;
    for xi := 0 to src.Width - 1 do
    begin
      if tileWidth < 2
      then DRows[yi, xi] := SRows[sy, xi]
      else begin
        sx := xm + xoffs;
        if   (sx >= 0) and (sx < (src.Width - 1))
        then DRows[yi, xi] := SRows[sy, sx]
        else DRows[yi, xi] := SRows[sy, xi];

        Inc (xoffs, 2);
        if xoffs >= tileWidth-1 then
        begin
          xm := xm + tileWidth;
          xoffs := xts;
        end;
      end;
    end;

    Inc (yoffs, 2);
    if yoffs >= tileHeight-1 then
    begin
      ym := ym + tileHeight;
      yoffs := yts;
    end;
  end;

  finally
    if tempBm <> nil then tempBm.Free;
    dst.Changed;
  end;
end; { Gimp_GlassTile32 }


//--- drawing routines ---

//*********************************************************
// select drawing destination bitmap
// input:      bitmap:    drawing destination bitmap
//---------------------------------------------------------
procedure SelectBitmap32 (bitmap: tBitmap32);
begin
{$ifdef test1}
  with bitmap do
  writeln (tst,'Gr32_Ext.SelectBitmap32: ', HexString(bitmap)
              , Width:6, Height:5);
{$endif}
  activeBitmap := bitmap;
end;
//*********************************************************
// draw polygon line or filled polygon
// input:      bitmap:    destination bitmap
//             points:    polygon points
//             color:     polygon line drawing color
//             closed:    polygon open or closed
//             antialiased:   =true: use antialaised line drawing
//             filled:    filled polygon or polygon line
//---------------------------------------------------------
procedure DrawPolygon (const bitmap: tBitmap32;
                       const points: tArrayOfFixedPoint;
                       const color: tColor32;
                       const closed: boolean;
                       const antialiased: boolean;
                       const filled: boolean);
begin
  if antiAliased then
    if filled
    then PolygonXS (bitmap, points, color)
    else PolyLineXS (bitmap, points, color, closed)
  else
    if filled
    then PolygonTS (bitmap, points, color)
    else PolyLineTS (bitmap, points, color, closed);
end;
//*********************************************************
// calculate floating point polygon points of a rotated ellipse
//  input:     mx, my: single       ellipse center
//             a,b: single          axis length
//             angle: float         rotation angle [degrees]
//             edges: cardinal      polygon edges
//             addPoints: cardinal  add points to existing curve
//             points: TArrayOfFixedPoint  polygon edge data
//---------------------------------------------------------
procedure CalcRotatedEllipsePoints (const mx,my: single;
                                    const a,b: single;
                                    const angle: single;
                                    const edges: cardinal;
                                    const addPoints: boolean;
                                    var points: TArrayOfFixedPoint);
var   mf, sinAngle, cosAngle, theta, xp,yp, xr,yr: single;
      li, pnts: integer;
begin
  Assert (edges >= 2, #13#10'edges must be >= 2'#13#10);
  mf := 2*Pi / edges;                  // multiplication factor
  sinAngle := sin (angle*deg);
  cosAngle := cos (angle*deg);
  if addPoints
  then begin                           // add points to existing curve
    pnts := High(points) + integer(edges) + 1;
    SetLength (points, pnts+1);
    for li := edges downto 0
    do begin
      theta := li * mf;                // angle step
      xp := a * cos(theta);            // ellipse point
      yp := b * sin(theta);

      // rotate ellipse point around center
      xr := mx - xp * cosAngle + yp * sinAngle;
      yr := my + xp * sinAngle + yp * cosAngle;
      points[pnts-li] := FixedPoint(xr, yr);  // add one floating point
    end;
  end else begin                       // calculate new ellipse points
    SetLength (points, edges+1);       // set polygon edges
    for li := 0 to edges
    do begin
      theta := li * mf;                // angle step
      xp := a * cos(theta);            // ellipse point
      yp := b * sin(theta);

      // rotate ellipse point around center
      xr := mx - xp * cosAngle + yp * sinAngle;
      yr := my + xp * sinAngle + yp * cosAngle;
      points[li] := FixedPoint(xr, yr);     // add one floating point
    end;
  end;
end;

//*********************************************************
// calculate floating point polygon points of a non rotated ellipse
//  input:     mx, my: single       ellipse center
//             a,b: single          axis length
//             edges: cardinal       polygon edges
//  output     points: TArrayOfFixedPoint  polygon edge data
//---------------------------------------------------------
procedure CalcEllipsePoints (const mx,my: single;
                             const a,b: single;
                             const edges: cardinal;
                             var points: TArrayOfFixedPoint);

var   i1: integer;   mf, px,py, angle: float;
begin
  Assert (edges >= 2, #13#10'edges must be >= 2'#13#10);
  SetLength (points, edges);           // set polygon edges
  mf := 6.28 / (edges-1);              // multiplication factor

  // calculate polygon data
  angle := 0;
  for i1 := 0 to edges - 1
  do begin
    px := mx + a * cos (angle);
    py := my + b * sin (angle);
    points[i1] := FixedPoint(px, py);  // add one point
    angle := angle + mf;
  end;
end;

//*********************************************************
//  draw ellipse as polygon curve
//  input:     mx, my: single       middle point
//             a,b: single          axis length
//             angle: float         rotation angle [degrees]
//             dc, fc: tColor32     line and fill color
//             edges: cardinal      polygon edges
//             antialiased:boolean  on / off
//             filled: boolean      filled polygon or polyline
//*********************************************************
procedure DrawEllipse (mx,my, a,b, angle: single;
                       color: tColor32;
                       edges: cardinal;
                       thickness: cardinal;
                       antialiased: boolean;
                       filled: boolean);

var   curve: TArrayOfFixedPoint;
begin
//  writeln (tst, 'Gr32_Ext.DrawEllipse: ', mx:8:2, my:8:2,  a:8:2, b:8:2, HexString (color):10);

  if thickness <= 1
  then begin
    if angle = 0.0
    then CalcEllipsePoints (mx,my, a,b, edges, curve)
    else CalcRotatedEllipsePoints (mx,my, a,b, angle, edges, false, curve);
    DrawPolygon (activeBitmap, curve, color, true, antialiased, filled);
  end else begin
    CalcRotatedEllipsePoints (mx,my, a-thickness,b-thickness, angle, edges, false, curve);
    CalcRotatedEllipsePoints (mx,my, a+thickness,b+thickness, angle, edges, true, curve);
    DrawPolygon (activeBitmap, curve, color, true, antialiased, true);
  end;
end;

//*********************************************************
// calculate polygon points of cubic curve through given points.
// input:      points:  polygon points of type tFixedPoint
//             steps:   interpolation steps between 2 polygon points
//             closed:  =true: closed curve, =false: open curve
// output:     curve:   spline curve with TFixedPoint vertices
//---------------------------------------------------------
procedure CalcCubicCurve (const points: tArrayOfFixedPoint;
                          const steps: cardinal;
                          const closed: boolean;
                          var curve: tArrayOfFixedPoint);

  function CubicPoint (v1, v2, v3, v4: integer; t: single): integer;
  begin
    result := round (v2 + t*((-v1 + v3) + t*((2*v1 - 2*v2 + v3 - v4)
                     + t*(-v1 + v2 - v3 + v4))));
  end;

var   ni,si,ci, hp,lc: integer;   p1,p2,p3,p4: tFixedPoint;
begin
//  writeln (tst, 'Gr32_Ext.CalcCubicCurve: steps=',steps,'  closed=', closed);
  hp := High(Points);
  if hp < 1 then exit;                 //===>
  if closed then begin                 // closed curve ?
    lc := hp+1;
    p2 := points[hp];
  end else begin                       // curve is not closed
    lc := hp;
    p2 := points[0];
  end;
  p3 := points[0];
  p4 := points[1];

  SetLength (curve, lc*integer(steps)+1);
  curve[0] := points[0];
  ci := 1;
  for ni := 1 to lc do
  begin
    p1 := p2;                          // shift curve points
    p2 := p3;
    p3 := p4;
    if closed
    then p4 := points[(ni+1)mod (hp+1)]
    else if ni+1 <= hp
    then p4 := points[ni+1];

    // calculate interpolation points
    for si := 1 to steps do begin
      curve[ci].x := CubicPoint (p1.x, p2.x, p3.x, p4.x, si / steps);
      curve[ci].y := CubicPoint (p1.y, p2.y, p3.y, p4.y, si / steps);
//  writeln (tst, ci:3, curve[ci].x shr 16 :7, curve[ci].y div 65536:7);
      inc (ci);
    end;
  end;
end;

//*********************************************************
// draw cubic spline 2d curve through given polygon points.
// features:   open or closed polygon curve
//             curve line with different curve line thickness
//             transparent curve line or area drawing
// input:      points:    curve points of type tFixedPoint
//             steps:     interpolation steps between 2 points
//             color:     curve color
//             closed:    curve open or closed
//             antiAliasing:   on / off
//             filled:    filled polygon or polyline
//             thickness: curve line thickness
//---------------------------------------------------------
procedure DrawCubicCurve (const points: tArrayOfFixedPoint;
                          const steps: cardinal;
                          const color: tColor32;
                          const closed: boolean;
                          const antialiased: boolean;
                          const filled: boolean;
                          const thickness: byte = 1);
var   curve: TArrayOfFixedPoint;
begin
  CalcCubicCurve (points, steps, closed, curve);
  if (thickness <= 1) or filled
  then //  draw polygon curve with thickness = 1
    DrawPolygon (activeBitmap, curve, color, closed, antialiased, filled)
  else begin
    // draw polygon curve with given thickness
    outline.Clear;
    outline.AddPoints (curve[0], length(curve));
    outline.FillMode := pfWinding;
    outline.Closed := closed;
    outline := Outline.Outline;
    outline := Outline.Grow(Fixed(thickness), 1);
    outline.Antialiased := antialiased;
    outline.DrawFill (activeBitmap, color);
  end
end;

//--- picture file routines ---

//*********************************************************
// save bitmap as standand windows BMP picture file.
// input:      bitmap       save this bitmap as picture file
//             filename     picture file name (*.jpg)
// result:     boolean      =true: picture saved
//---------------------------------------------------------
function SaveBitmapToBMP (const bitmap: tBitmap32;
                          const filename: string): boolean;
var    bm: tBitmap;
begin
{$ifdef test2}
  writeln (tst, 'SaveBitmapToBMP: ', filename);
{$endif}

  result := false;
  if bitmap = nil then exit;
  try
    bm := Graphics.TBitmap.Create;
    try
      bm.Assign (bitmap);              // convert bitmap32 into bitmap
      bm.SaveToFile (filename);
      result := true;
    finally
      bm.Free;
    end;
  except
  end;
end;

//*********************************************************
// save bitmap as JPEG picture file.
// input:      bitmap       save this bitmap as picture file
//             filename     picture file name (*.jpg)
//             quality      picture quality in % (100=best)
//             progEncoding =true: progressive encoding
// result:     boolean      =true: picture saved as picture file
//---------------------------------------------------------
function SaveBitmapToJPG (const bitmap: tBitmap;
                          const filename: string;
                          const quality: integer = 95;
                          const progEncoding: boolean = false): boolean;
var    jpg: TJPEGImage;
begin
{$ifdef test2}
  writeln (tst, 'SaveBitmapToJPG: ', filename);
{$endif}

  result := false;
  if bitmap = nil then exit;
  jpg := nil;
  try
    jpg := TJpegImage.Create;
    try
      jpg.Assign (bitmap);             // copy bitmap to jpeg
      jpg.CompressionQuality := quality;
      jpg.Compress;                    // compress data
      jpg.ProgressiveEncoding := progEncoding;
      jpg.SaveToFile (filename);
      result := true;
    except
    end;
  finally
    jpg.Free;
  end;
end;

//*********************************************************
// save bitmap32 as JPEG picture file.
// input:      bitmap       save this bitmap as picture file
//             filename     picture file name (*.jpg)
//             quality      picture quality in % (100=best)
//             progEncoding =true: progressive encoding
// result:     boolean      =true: picture saved
//---------------------------------------------------------
function SaveBitmap32ToJPG (const bitmap: tBitmap32;
                            const filename: string;
                            const quality: integer = 95;
                            const progEncoding: boolean = false): boolean;
var    bm: tBitmap;
begin
{$ifdef test2}
  writeln (tst, 'SaveBitmap32ToJPG: ', filename);
{$endif}

  bm := nil;
  try
    bm := Graphics.TBitmap.Create;
    bm.Assign (bitmap);              // convert bitmap32 into bitmap
    result := SaveBitmapToJPG (bm, filename, quality, progEncoding);
  finally
    bm.Free;
  end;
end;

//*********************************************************
// write rectangle data to given text file
// input:      out: text      output file
//             name: string   name of rectangle
//---------------------------------------------------------
(*
{$ifopt D+}
procedure DisplayRectangle (var out: text; name: string; rect: tRect);
begin
  with rect
  do writeln (tst, name,':  ', Left:6, top:6, right:6, bottom:6,
                   '  w=',right-left+1:6, '  h=', bottom-top+1:6);
end;
{$endif}
*)
//*********************************************************
// get inner B rectangle
// input:      outerA   outer rectangle A
//             innerA   inner rectangle A
//             sizeA    outer size B
// output:     sizeB    inner size B
// result:     boolean  =true: inner B size calculated
//*********************************************************
function GetScaledRect (const outerA, innerA: tRect;  sizeA: TPoint;
                        var sizeB: TPoint): boolean;
var   ax,ay, bx,by: Integer;
begin
  with outerA do begin
    ax := Right - Left + 1;   ay := Bottom - Top + 1;
  end;
  with innerA do begin
    bx := Right - Left;   by := Bottom - Top;
    result := not ((bx <= 0) or (by <= 0));
    if not result then begin
      FillChar (sizeB, sizeof(TRect), #0);
      exit;
    end;
  end;
  sizeB.X := bx * sizeA.X div ax;
  sizeB.Y := by * sizeA.Y div ay;
end;
//*********************************************************
// get rectangle (rect) inside another rectangle (dst)
// preserving aspect ratio of a given rectangle (src)
// input:      src      source rectangle
//             dst      destination rectangle
//             position rectangle position      1  2  3
//                                              4  5  6
//                                              7  8  9
// output:     rect     src rectangle scaled to dst rectangle
// result:     boolean  =true: rectangle calculated
//*********************************************************
function GetFittingRect (const src, dst: TRect;
                         const position: word;
                         var rect: TRect): boolean;
var   srcWidth, srcHeight, dstWidth, dstHeight, ScaledSide: Integer;
begin
  result := false;
  with src do begin
    srcWidth := Right - Left;
    srcHeight := Bottom - Top;
  end;
  if (srcWidth = 0) or (srcHeight = 0) then exit;

  with dst do begin
    dstWidth := Right - Left;
    dstHeight := Bottom - Top;
  end;
  if (dstWidth = 0) or (dstHeight = 0) then exit;

  if (srcWidth / srcHeight) > (dstWidth / dstHeight)
  then begin                           // full width         // +===+
    ScaledSide := Round(dstWidth * srcHeight / srcWidth);    // +   +
    case position of                                         // +===+
      1,2,3: rect.Top := dst.Top;                                   // top
      7,8,9: rect.Top := dst.Bottom - ScaledSide;                   // buttom
      else   rect.Top := dst.Top + (dstHeight - ScaledSide) div 2;  // center
    end;
    rect.Bottom := rect.Top + ScaledSide;
    rect.Left := dst.Left;
    rect.Right := dst.Right;
  end else begin                       // full height
    ScaledSide := Round(dstHeight * srcWidth / srcHeight);   // ++--++
    rect.Top := dst.Top;                                     // ||  ||
    rect.Bottom := dst.Bottom;                               // ++--++
    case position of
      1,4,7: rect.Left := dst.Left;                                  // left
      3,6,9: rect.Left := dst.Right - ScaledSide;                    // right
      else   rect.Left := dst.Left + (dstWidth - ScaledSide) div 2;  // center
    end;
    rect.right := rect.left + ScaledSide;
  end;
  result := true;
end;
//*********************************************************
// print given Bitmap with given picture title
// input:     bitmap      bitmap to print
//            title       picture title
//            printTitle  =true: printTitle
//            frame       frame size in percent [%] (optional)
//---------------------------------------------------------
procedure PrintBitmap32 (const bitmap: tBitmap32;
                         const title: string;
                         const printTitle: boolean;
                         const position: word;
                         const frame: word = 5);

var   prX, prY,  frX, frY: Integer;
      srcRect, prnRect, dstRect: TRect;
begin
  Printer.Title := title;
  prX := Printer.PageWidth;
  prY := Printer.PageHeight;
  frX := prX * frame div 100;
  frY := prY * frame div 100;
  srcRect := Rect (0, 0, bitmap.Width, bitmap.Height);    // bitmap rectangle
  prnRect := Rect (frx, fry, prX-frX, prY-fry);           // printer rectangle
  if GetFittingRect (srcRect, prnRect, position, dstRect) // centered rectangle
  then try
//    DisplayRectangle (tst, 'src', srcRect);
//    DisplayRectangle (tst, 'prn', prnRect);
//    DisplayRectangle (tst, 'dst', dstRect);
    Printer.BeginDoc;
    // copy bitmap to printer.canvas
    // bitmap.StretchFilter := sfLinear;                   // v1.7
    SelectBitmapResampling  (bitmap, sfSplineKernel);      // >= v1.8

    // print picture title
    if printTitle and (Printer.Title <> '')
    then with Printer.Canvas do begin
      Font.Color := clblack;
      frY := dstRect.Top - 2 * abs(Font.Height);
      if frY < 60 then frY := 60;
      TextOut(dstRect.Left, frY, Printer.Title);
    end;

    // kopiere bitmap -> printer.bitmap
    bitmap.DrawTo (Printer.Canvas.Handle, dstRect, srcRect);

  finally
    Printer.EndDoc;
  end;
end;

//*********************************************************
// copy tile with position 0,0 and size dx,dy over the whole bitmap
// input:      dx,dy: tile size
//             bitmap: source and destination bitmap
//---------------------------------------------------------
procedure CopyTiles (dx,dy: integer; bitmap: tBitmap32);

var  xi,yi, width: integer;   srcRect: tRect;
begin
{$ifdef test2}
  writeln (tst, 'CopyTiles: x=',dx,' y=',dy);
{$endif}
  Assert ((dx > 0) and (dy > 0), #13#10'invalid tile size'#13#10);

  // fill horizontal tiles
  srcRect := Rect(0, 0, dx, dy);
  width := bitmap.width;
  xi := dx;
  while xi < width
  do begin
    bitmap.DrawTo (bitmap, Rect(xi, 0, xi+dx, dy), srcRect);
    xi := xi + dx;
  end;

  // fill vertical tiles
  srcRect := Rect(0, 0, bitmap.width, dy);
  yi := dy;
  while yi < bitmap.height
  do begin
    bitmap.DrawTo (bitmap, Rect(0, yi, width, yi+dy), srcRect);
    yi := yi + dy;
  end;
end;
//*********************************************************
// fill destination bitmap with given tile bitmap at offset x,y
// input:      src: source tile bitmap
//             dst: destination bitmap
//             x,y: left top destination position
//---------------------------------------------------------
procedure DrawTiles (src, dest: tBitmap32; x,y: integer);

var  xi, srcW, destW, srcH, destH: integer;   
begin
{$ifdef test2}
  writeln (tst, 'DrawTiles: x=',x,' y=',y);
{$endif}
  Assert ((src.width > 0) and (src.height > 0), #13#10'invalid bitmap32 size'#13#10);

  // find left top starting point
  while x > 0 do x := x - src.width;
  while y > 0 do y := y - src.height;
  srcW  := src.width;
  destW := dest.width;
  srcH  := src.height;
  destH := dest.height;
  // wegen scrolling wird jedes tile extra gezeichnet !
  while y < destH
  do begin
    xi := x;
    while xi < destW
    do begin
      src.DrawTo (dest, xi,y);
      xi := xi + srcW;
    end;
    y := y + srcH;
  end;
end;
//*********************************************************
// fill destination bitmap with tiles scaled to rect before
// input:      src: source bitmap
//             dst: destination bitmap
//             rect: destination rectangle
//---------------------------------------------------------
procedure DrawScaledTiles (src, dest: tBitmap32; rect: tRect);
var  w,h: integer;
     tempBm: tBitmap32;   tempRect: tRect;
begin
  tempBm := tBitmap32.Create;
  w := rect.Right - rect.Left;
  h := rect.Bottom - rect.Top;
  tempBm.SetSize (w, h);
  FillChar (tempRect, sizeof(tempRect), 0);
  tempRect.Right := w;
  tempRect.Bottom := h;
  src.DrawTo (tempBm, tempRect);
//  dest.Clear; //test
  DrawTiles (tempBm, dest, rect.left, rect.top);
  tempBm.Free;
end;
//*********************************************************
// draw chessboard pattern with given size and colors
// input:      bm: tBitmap32;         draw to this bitmap
//             dx,   dy: integer;     tile size
//             col1, col2: color32;   fill colors
//---------------------------------------------------------
procedure DrawChessboard (bm: tBitmap32; dx,dy: integer; c1,c2: tColor32);
begin
  with bm do begin
    FillRectTS (0,0,dx,dy, c1);
    FillRectTS (dx,dy,2*dx,2*dy, c1);
    FillRectTS (dx,0,2*dx,dy, c2);
    FillRectTS (0,dy,dx,2*dy, c2);
  end;
  CopyTiles (2*dx,2*dy, bm);
end;

//*********************************************************
// fill image background with chess board texture
// input:      img: TImgView32;       draw to this image
//             dx,   dy: integer;     tile size
//             col1, col2: color32;   fill colors
//---------------------------------------------------------
procedure ImageChessboard (img: TImgView32;
                           dx,dy: integer;
                           col1, col2: tColor32);
var Colors: array [0..1] of TColor32;
var
  W, ni, parity: Integer;
  line1, line2: TArrayOfColor32; // a buffer for a couple of scanlines
begin
  colors[0] := col1;
  colors[1] := col2;
  with img.Buffer do begin
    W := Width;
    SetLength(Line1, W);
    SetLength(Line2, W);
    for ni := 0 to W - 1 do
    begin
      Parity := ni shr 3 and $1;
      Line1[ni] := Colors[Parity];
      Line2[ni] := Colors[1 - Parity];
    end;
    for ni := 0 to Height - 1 do
    begin
      Parity := ni shr 3 and $1;
      if Boolean(Parity)
      then MoveLongword(Line1[0], ScanLine[ni]^, W)
      else MoveLongword(Line2[0], ScanLine[ni]^, W);
    end;
  end;
end;

//--- tPalette32 routines ---

//*********************************************************
// Interpolation der Farben von Farbindex first bis last
// in der bergebenen Farbpalette.
// Versorgung:  pal: tPalette;      Farbpalette
//              first, last: tColorRGBI;  1..letzte Farbe
// Ergebnis:    pal: tPalette;      neue Farbpalette
//---------------------------------------------------------
procedure SpreadColor32 (var pal: tArrayOfColor32; first, last: tColorRGBA);

var   ci,dc: word;
      rd,gd,bd, rv,gv,bv: longint;   act: tColorBGRA;
begin
{$ifdef test1}
  writeln (tst, 'FractMap.SpreadColor: ', first.a, '  ',last.a);
{$endif}

  dc := last.a - first.a + 1;
  rd := last.r - first.r;   rv := 0;
  gd := last.g - first.g;   gv := 0;
  bd := last.b - first.b;   bv := 0;
  act.a := $FF;
  for ci := first.a + 1 to last.a
  do begin
    inc (rv,rd);   act.r := first.r + rv div dc;
    inc (gv,gd);   act.g := first.g + gv div dc;
    inc (bv,bd);   act.b := first.b + bv div dc;
    pal[ci] := TColor32(act);
  end;
end;
//*********************************************************
// create new color palette given by spread color array
// input:      pal             tColor32 palette
//             spreadValues:   spread color array
//---------------------------------------------------------
procedure SetSpreadPalette32 (var pal: TArrayOfColor32;
                              var spreadValues: array of tColorRGBA);
var   ci, first, last: word;
begin
  last := high(spreadValues);
  SetLength (pal, spreadValues[last].a + 1);
  first := spreadValues[0].a;
  pal[first] := tColor32(spreadValues[0]);
  for ci := 1 to last
  do SpreadColor32 (pal, spreadValues[ci-1], spreadValues[ci]);
end;
//*********************************************************
initialization
  // create 32 bit work line
  hLine32Bm := TBitmap32.Create;
  hLine32Bm.SetSize (2048, 11);

  outline := tPolygon32.Create;

//*********************************************************
finalization
  hLine32Bm.Free;
  Outline.Free;

end.

