
//*********************************************************
//
//  SSDemo2.dpr       Graphics32 Screen Saver demo
//
//  delphi 5/6/7 code by Gerd Platl
//
//  this program is also used as test frame for gr32 draw primitives
//
//-------- start parameters -------------------------------
//  /a     screen saver password input (not implemented)
//  /c     screen saver configuration
//  /p pwh screen saver preview pwh=parent window handle
//  /s     screen saver start
//
//-------- input ------------------------------------------
//  right mouse button  show configuration window
//  [WinTest]           show test window
//                        exit only at close button
//  [Test]              show full screen demo
//                        exit on any input event
//  [Cancel]            exit without saving parameters
//  [OK]                save parameters and exit
//
//-------- history ----------------------------------------
//  v2.0  29.Jan.2002  first release, delphi 5
//  v2.1  23.May 2004  delphi-6 version
//  v2.2  01.Mar 2005  splines added (graphics32 v1.7)
//  v2.3  06.Nov 2005  small changes (graphics32 v1.8)
//  v2.4  07.Nov 2006  using new versions of unit files (graphics32 v1.8.1)
//
//*********************************************************
program SSDemo2;

{$ifopt D+}
  {$define DEBUG}
{$endif}

{%File 'comp_opt.inc'}
{%File 'float.inc'}
{%File 'Colors32.inc'}

uses
  SysUtils,
  Forms,
  Windows,
  S_Screen in 'S_Screen.PAS' {ScreenForm},
  S_Config in 'S_Config.PAS' {ConfigForm},
  SysUtil2 in 'SysUtil2.pas',
  Gr32_Ext in 'Gr32_Ext.pas',
  Gr32_Gouraud in 'Gr32_Gouraud.pas',
  ColorRGB in 'ColorRGB.pas';

{$E scr}

{$D Graphics32 Screen Saver Example}

type ssFunction = (ssDebug, ssConfig, ssStart, ssPreview, ssPassword);

var  start: ssFunction = ssDebug;
     parentWindowHandle: tHandle = 0;
     param1: string = '';
     param2: string = '';

//---------------------------------------------------------
procedure GetStartMode;
begin
  start := ssDebug;      // default starting mode
  if (ParamCount > 0)
  then begin
    param1 := UpperCase(ParamStr(1));
    param2 := ParamStr(2);
    if param2 <> ''
    then parentWindowHandle := StrToIntDef(param2,-1);
    if (param1[1] = '-')
    or (param1[1] = '/')
    then case param1[2] of
      'A': start := ssPassword;
      'C': start := ssConfig;
      'P': start := ssPreview;
      'S': start := ssStart;
    end;
  end;
//  writeln (tst, 'start=',ord(start));
end;
//---------------------------------------------------------
procedure PrepareScreenSaver;
begin
  GetStartMode;

  SetWindowLong(Application.Handle, GWL_EXSTYLE, WS_EX_TOOLWINDOW);

  case start of
    ssDebug,
    ssConfig: begin     {configForm needs to be the main form}
      Application.CreateForm(TconfigForm, configForm);
  Application.CreateForm(TscreenForm, screenForm);
  end;

    ssStart: begin
//      CmdShow := SW_SHOWMAXIMIZED;
      Application.CreateForm(tScreenForm, screenForm);
      Application.CreateForm(tConfigForm, configForm);
      ScreenForm.SetBounds (0,0, startScreenInfo.x, startScreenInfo.y);
    end;

    ssPreview: begin   {screenForm needs to be the main form}
      Application.CreateForm(tScreenForm, screenForm);
      Application.CreateForm(tConfigForm, configForm);
      if parentWindowHandle = 0
      then begin
        ConfigForm.Show;
        parentWindowHandle := configForm.Handle;
      end;
      ScreenForm.SetParentWindow (parentWindowHandle);
    end;

    // ssPassword: begin
    //   if PasswordRequired Then PasswordForm.Show;
    // end;
  end;
  configForm.Caption := configForm.Caption + param1;
end;
//---------------------------------------------------------
procedure ExitMultiStartedProgram;
begin
  if InitMutex (programName + copy(paramStr(1),1,2))
  then Halt (1);
end;
//---------------------------------------------------------
begin
  ExitMultiStartedProgram;   // Only one instance is allowed at one time!

  PrepareScreenSaver;
  Application.Title := 'Graphics32 Screen Saver Demo 2';
  Application.Run;
end.

