
//*********************************************************
//
//  UNIT:     S _ C O N F I G     29.01.2002,  12.09.2005
//
//  ScreenSaver Configuration handling:
//  Load, Save and Edit Configuration and test ScreenSaver
//
//*********************************************************
unit S_CONFIG;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons,
  Gr32, GR32_RangeBars, ExtCtrls;

type
  tConfigForm = class(TForm)
    Panel1: TPanel;
    GroupBox1: TGroupBox;
      iLines: TCheckBox;
      iBars: TCheckBox;
      iTriangles: TCheckBox;
      iFillTria: TCheckBox;
      iSplineCurve: TCheckBox;
      iSplineArea: TCheckBox;
      iEllipse: TCheckBox;
    GroupBox2: TGroupBox;
      bLines: TCheckBox;
      bBars: TCheckBox;
      bTriangles: TCheckBox;
      bFillTria: TCheckBox;
      bSplineCurve: TCheckBox;
      bSplineArea: TCheckBox;
      bEllipseDisk: TCheckBox;
    GroupBox3: TGroupBox;
      Label1: TLabel;
      Label2: TLabel;
      Label3: TLabel;
      Label4: TLabel;
      Label5: TLabel;
      Label6: TLabel;
      iObjects: TLabel;
      gObjects: TGaugeBar;
      iSize: TLabel;
      gSize: TGaugeBar;
      iAlpha: TLabel;
      gAlpha: TGaugeBar;
      iThick: TLabel;
      gThick: TGaugeBar;
      iSteps: TLabel;
      gSteps: TGaugeBar;
    GroupBox5: TGroupBox;
      bgBars: TCheckBox;
      bgTriangles: TCheckBox;
    GroupBox4: TGroupBox;
      gBars: TCheckBox;
      gTriangles: TCheckBox;
    LabelVersion: TLabel;
    LabelGr32Version: TLabel;
    WinTestButton: TButton;
    TestButton: TBitBtn;
    ClearButton: TButton;
    PauseButton: TButton;
    CancelButton: TBitBtn;
    OKButton: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);

    procedure TestButtonClick(Sender: TObject);
    procedure WinTestButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure PauseButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure gObjectsChange(Sender: TObject);
    procedure gSizeChange(Sender: TObject);
    procedure gAlphaChange(Sender: TObject);
    procedure gThicknessChange(Sender: TObject);
    procedure gStepsChange(Sender: TObject);
    procedure iMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    mButton: TMouseButton;
    { Private declarations }
    procedure LoadConfig;
    procedure SaveConfig;
  end;

var
  configForm: tConfigForm;

implementation                        

{$R *.DFM}

uses  IniFiles, S_Screen;

const configFile = 'SSDEMO2.INI';
      secCf = 'Config';
      secAA = 'AntiAliasing';
      secBl = 'Blend';
      secGo = 'Gouraud';
      secBG = 'BlendGouraud';

// load configuration data from SSDEMO2.INI file
procedure tConfigForm.LoadConfig;
var  inifile: TIniFile;
begin
  inifile := TIniFile.Create(configFile);
  try
    with inifile do begin
      gObjects.position    := ReadInteger(secCf, 'Objects',  50);
      gSize.position       := ReadInteger(secCf, 'Size',     80);
      gAlpha.position      := ReadInteger(secCf, 'Alpha',   127);
      gThick.position      := ReadInteger(secCf, 'Thickness', 1);
      gSteps.position      := ReadInteger(secCf, 'Steps',    10);
      iObjects.Caption     := IntToStr (gObjects.Position);
      iSize.Caption        := IntToStr (gSize.Position);
      iAlpha.Caption       := IntToStr (gAlpha.Position);
      iThick.Caption       := IntToStr (gThick.Position);
      iSteps.Caption       := IntToStr (gSteps.Position);

      iLines.Checked       := ReadBool(secAA, 'Lines',       true);
      iBars.Checked        := ReadBool(secAA, 'Bars',        false);
      iTriangles.Checked   := ReadBool(secAA, 'Triangles',   false);
      iFillTria.Checked    := ReadBool(secAA, 'FillTria',    false);
      iSplineCurve.Checked := ReadBool(secAA, 'SplineCurve', false);
      iSplineArea.Checked  := ReadBool(secAA, 'SplineArea',  false);
      iEllipse.Checked     := ReadBool(secAA, 'Ellipse',     false);

      bLines.Checked       := ReadBool(secBl, 'Lines',       false);
      bBars.Checked        := ReadBool(secBl, 'Bars',        true);
      bTriangles.Checked   := ReadBool(secBl, 'Triangles',   false);
      bFillTria.Checked    := ReadBool(secBl, 'FillTria',    false);
      bSplineCurve.Checked := ReadBool(secBl, 'SplineCurve', false);
      bSplineArea.Checked  := ReadBool(secBl, 'SplineArea',  false);
      bEllipseDisk.Checked := ReadBool(secBl, 'EllipseDisk', false);

      gBars.Checked        := ReadBool(secGo, 'Bars',        false);
      gTriangles.Checked   := ReadBool(secGo, 'Triangles',   false);

      bgBars.Checked       := ReadBool(secBG, 'Bars',        false);
      bgTriangles.Checked  := ReadBool(secBG, 'Triangles',   false);
    end;
  finally
    inifile.Free;
  end;
end;

// save configuration data to SSDEMO2.INI file
procedure tConfigForm.SaveConfig;
var   inifile: TIniFile;
begin
  inifile := TIniFile.Create(configFile);
  try
    with inifile do begin
      WriteInteger (secCf, 'Objects',  gObjects.position);
      WriteInteger (secCf, 'Size',     gSize.position);
      WriteInteger (secCf, 'Alpha',    gAlpha.position);
      WriteInteger (secCf, 'Thickness',gThick.position);
      WriteInteger (secCf, 'Steps',    gSteps.position);

      WriteBool    (secAA, 'Lines',       iLines.Checked);
      WriteBool    (secAA, 'Bars',        iBars.Checked);
      WriteBool    (secAA, 'Triangles',   iTriangles.Checked);
      WriteBool    (secAA, 'FillTria',    iFillTria.Checked);
      WriteBool    (secAA, 'SplineCurve', iSplineCurve.Checked);
      WriteBool    (secAA, 'SplineArea',  iSplineArea.Checked);
      WriteBool    (secAA, 'Ellipse',     iEllipse.Checked);

      WriteBool    (secBl, 'Lines',       bLines.Checked);
      WriteBool    (secBl, 'Bars',        bBars.Checked);
      WriteBool    (secBl, 'Triangles',   bTriangles.Checked);
      WriteBool    (secBl, 'FillTria',    bFillTria.Checked);
      WriteBool    (secBl, 'SplineCurve', bSplineCurve.Checked);
      WriteBool    (secBl, 'SplineArea',  bSplineArea.Checked);
      WriteBool    (secBl, 'EllipseDisk', bEllipseDisk.Checked);

      WriteBool    (secGo, 'Bars',        gBars.Checked);
      WriteBool    (secGo, 'Triangles',   gTriangles.Checked);

      WriteBool    (secBG, 'Bars',        bgBars.Checked);
      WriteBool    (secBG, 'Triangles',   bgTriangles.Checked);
    end;
  finally
    inifile.Free;
  end;
end;

procedure tConfigForm.FormCreate(Sender: TObject);
begin
  LoadConfig;
  LabelGr32Version.Caption := 'gr32 v' + Graphics32Version;
end;

procedure tConfigForm.FormActivate(Sender: TObject);
begin
  top := 40;
  if ScreenForm.IsPaused
  then PauseButton.Caption := '&Run'
  else PauseButton.Caption := '&Pause';
end;

//ToDo: not called - why ???
procedure tConfigForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  case key of
    'p': ScreenForm.TogglePaused;  // Pause on/off
  end;
end;

procedure tConfigForm.TestButtonClick(Sender: TObject);
begin
  ScreenForm.Show;
  ScreenForm.Restart (false);
  ScreenForm.Show;
end;

procedure tConfigForm.WinTestButtonClick(Sender: TObject);
begin
  ScreenForm.Restart (true);
  ScreenForm.Show;
end;

procedure tConfigForm.ClearButtonClick(Sender: TObject);
begin
  ScreenForm.Clear;
end;

procedure tConfigForm.PauseButtonClick(Sender: TObject);
begin
  ScreenForm.TogglePaused;             // Pause on/off
  if ScreenForm.IsPaused
  then PauseButton.Caption := '&Run'
  else PauseButton.Caption := '&Pause';
end;

procedure tConfigForm.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

procedure tConfigForm.OKButtonClick(Sender: TObject);
begin
  SaveConfig;
  Close;
end;

procedure tConfigForm.gObjectsChange(Sender: TObject);
begin
  iObjects.Caption := IntToStr (gObjects.Position);
end;

procedure tConfigForm.gSizeChange(Sender: TObject);
begin
  iSize.Caption := IntToStr (gSize.Position);
end;

procedure tConfigForm.gAlphaChange(Sender: TObject);
begin
  iAlpha.Caption := IntToStr (gAlpha.Position);
end;

procedure tConfigForm.gThicknessChange(Sender: TObject);
begin
  iThick.Caption := IntToStr (gThick.Position);
end;

procedure tConfigForm.gStepsChange(Sender: TObject);
begin
  iSteps.Caption := IntToStr (gSteps.Position);
end;

procedure tConfigForm.iMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  var alabel: tLabel;
begin
  mButton := button;
  alabel := tLabel(sender);
  case mButton of

    mbLeft:    // set average value
      case alabel.Tag of
        1: gObjects.Position := 500;
        2: gSize.Position := 55;
        3: gAlpha.Position := 127;
        4: gThick.Position := 5;
        5: gSteps.Position := 8;
      end;

    mbRight:   // set max value
      case alabel.Tag of
        1: gObjects.Position := gObjects.max;
        2: gSize.Position := gSize.max;
        3: gAlpha.Position := gAlpha.max;
        4: gThick.Position := gThick.max;
        5: gSteps.Position := gSteps.max;
      end;
  end;
end;

end.

